@@ -1,4 +1,5 @@
 require "rbconfig"
+require "shellwords"
 
 require 'rake/baseextensiontask'
 require "rubygems/package_task"
@@ -79,6 +80,26 @@ module Rake
       end.flatten
     end
 
+    def make_makefile_cmd(root_path, tmp_path, extconf, cross_platform) # :nodoc:
+      # include current directory
+      include_dirs = ['.'].concat(@config_includes).uniq.join(File::PATH_SEPARATOR)
+
+      # build a relative path to extconf script
+      abs_tmp_path = (Pathname.new(root_path) + tmp_path).realpath
+      abs_extconf = (Pathname.new(root_path) + extconf).realpath
+      rel_extconf = abs_extconf.relative_path_from(abs_tmp_path).to_s
+
+      # base command
+      cmd = [Gem.ruby, "-I#{include_dirs}", rel_extconf]
+
+      # add all the options
+      cmd += @config_options
+      cmd += cross_config_options(cross_platform) if cross_platform
+      cmd += extra_options
+
+      cmd.compact
+    end
+
     private
     # copy other gem files to staging directory
     def define_staging_file_tasks(files, lib_path, stage_path, platf, ruby_ver)
@@ -120,7 +141,6 @@ module Rake
       tmp_path = "#{@tmp_dir}/#{platf}/#{@name}/#{ruby_ver}"
       stage_path = "#{@tmp_dir}/#{platf}/stage"
 
-      siteconf_path = "#{tmp_path}/.rake-compiler-siteconf.rb"
       tmp_binary_path = "#{tmp_path}/#{binary_path}"
       tmp_binary_dir_path = File.dirname(tmp_binary_path)
       stage_binary_path = "#{stage_path}/#{lib_binary_path}"
@@ -138,26 +158,20 @@ module Rake
       directory lib_path
       directory stage_binary_dir_path
 
-      directory File.dirname(siteconf_path)
-      # Set paths for "make install" destinations
-      file siteconf_path => File.dirname(siteconf_path) do
-        File.open(siteconf_path, "w") do |siteconf|
-          siteconf.puts "require 'rbconfig'"
-          siteconf.puts "require 'mkmf'"
-          siteconf.puts "dest_path = mkintpath(#{File.expand_path(lib_path).dump})"
-          %w[sitearchdir sitelibdir].each do |dir|
-            siteconf.puts "RbConfig::MAKEFILE_CONFIG['#{dir}'] = dest_path"
-            siteconf.puts "RbConfig::CONFIG['#{dir}'] = dest_path"
-          end
-        end
-      end
-
       # copy binary from temporary location to final lib
       # tmp/extension_name/extension_name.{so,bundle} => lib/
       task "copy:#{@name}:#{platf}:#{ruby_ver}" => [lib_path, tmp_binary_path, "#{tmp_path}/Makefile"] do
         # install in lib for native platform only
         unless for_platform
-          sh "#{make} install target_prefix=", chdir: tmp_path
+          relative_lib_path = Pathname(lib_path).relative_path_from(Pathname.new(tmp_path))
+
+          make_command_line = Shellwords.shellsplit(make)
+          make_command_line << "install"
+          make_command_line << "sitearchdir=#{relative_lib_path}"
+          make_command_line << "sitelibdir=#{relative_lib_path}"
+          make_command_line << "target_prefix="
+
+          sh(*make_command_line, chdir: tmp_path)
         end
       end
       # copy binary from temporary location to staging directory
@@ -187,37 +201,17 @@ Java extension should be preferred.
 
       # makefile depends of tmp_dir and config_script
       # tmp/extension_name/Makefile
-      file "#{tmp_path}/Makefile" => [tmp_path, extconf, siteconf_path] do |t|
-        options = @config_options.dup
-
-        # include current directory
-        include_dirs = ['.'].concat(@config_includes).uniq.join(File::PATH_SEPARATOR)
-        cmd = [Gem.ruby, "-I#{include_dirs}", "-r#{File.basename(siteconf_path)}"]
-
-        # build a relative path to extconf script
-        abs_tmp_path = (Pathname.new(Dir.pwd) + tmp_path).realpath
-        abs_extconf = (Pathname.new(Dir.pwd) + extconf).realpath
-
-        # now add the extconf script
-        cmd << abs_extconf.relative_path_from(abs_tmp_path)
-
-        # fake.rb will be present if we are cross compiling
-        if t.prerequisites.include?("#{tmp_path}/fake.rb") then
-          options.push(*cross_config_options(platf))
+      file "#{tmp_path}/Makefile" => [tmp_path, extconf] do |t|
+        if t.prerequisites.include?("#{tmp_path}/fake.rb")
+          cross_platform = platf
+        else
+          cross_platform = nil
         end
 
-        # add options to command
-        cmd.push(*options)
-
-        # add any extra command line options
-        unless extra_options.empty?
-          cmd.push(*extra_options)
-        end
+        command = make_makefile_cmd(Dir.pwd, tmp_path, extconf, cross_platform)
 
         chdir tmp_path do
-          # FIXME: Rake is broken for multiple arguments system() calls.
-          # Add current directory to the search path of Ruby
-          sh cmd.join(' ')
+          sh(*command)
         end
       end
 
@@ -519,12 +513,15 @@ Java extension should be preferred.
     def find_make
       candidates = ["gmake", "make"]
       paths = (ENV["PATH"] || "").split(File::PATH_SEPARATOR)
+      paths = paths.collect do |path|
+        Pathname(path).cleanpath
+      end
 
       exeext = RbConfig::CONFIG["EXEEXT"]
       candidates.each do |candidate|
         paths.each do |path|
-          make = File.join(path, "#{candidate}#{exeext}")
-          return make if File.executable?(make)
+          make = path + "#{candidate}#{exeext}"
+          return make.to_s if make.executable?
         end
       end
 
