@@ -58,6 +58,15 @@ describe Rake::ExtensionTask do
         end
         ext.platform.should == 'universal-foo-bar-10.5'
       end
+
+      it 'should allow extra sources to be added' do
+        ext = Rake::ExtensionTask.new('extension_one') do |ext|
+          ext.extra_sources << 'extra.c'
+        end
+        ext.extra_sources.should include('extra.c')
+        # Private API between the base task and the extension task
+        ext.send(:source_files).should include('extra.c')
+      end
     end
   end
 
@@ -251,7 +260,7 @@ describe Rake::ExtensionTask do
         end
 
         it 'should define as task for pure ruby gems' do
-          Rake::Task.task_defined?('native').should == false
+          Rake::Task.task_defined?('native').should be false
           Rake::ExtensionTask.new('extension_one', @spec)
           Rake::Task.task_defined?('native').should == true
         end
@@ -276,6 +285,98 @@ describe Rake::ExtensionTask do
       end
     end
 
+    context 'make_makefile_cmd' do
+      around :each do |test|
+        @tmp_path = "injectedtmp"
+        @extconf = "injectedextconf.rb"
+
+        Dir.mktmpdir do |root_path|
+          @root_path = root_path
+          FileUtils.mkdir_p("#{root_path}/#{@tmp_path}")
+          FileUtils.touch("#{root_path}/#{@extconf}")
+
+          @ext = Rake::ExtensionTask.new('extension_test') do |ext|
+            # repeats to test deduplication
+            ext.config_includes += ["/injected/include1", "/injected/include1", "/injected/include2"]
+            ext.config_options << "--with-a"
+            ext.extra_options = ["--", "--with-b"] # normally set based on ARGV
+            ext.cross_config_options << '--with-c'
+            ext.cross_config_options << {'universal-known' => '--with-d'}
+            ext.cross_config_options << {'universal-unknown' => '--with-e'}
+          end
+
+          Dir.chdir(root_path) do
+            test.run
+          end
+        end
+      end
+
+      it "runs the extconf with proper arguments when not cross-compiling" do
+        command = @ext.make_makefile_cmd(@root_path, @tmp_path, @extconf, nil)
+
+        expected_includes = [".", "/injected/include1", "/injected/include2"].join(File::PATH_SEPARATOR)
+        expected = [
+          Gem.ruby,
+          "-I#{expected_includes}",
+          "../#{@extconf}",
+          "--with-a", # config_options
+          "--", "--with-b", # extra_options
+        ]
+
+        command.should eq(expected)
+      end
+
+      it "runs the extconf with proper arguments when cross-compiling" do
+        command = @ext.make_makefile_cmd(@root_path, @tmp_path, @extconf, "universal-known")
+
+        expected_includes = [".", "/injected/include1", "/injected/include2"].join(File::PATH_SEPARATOR)
+        expected = [
+          Gem.ruby,
+          "-I#{expected_includes}",
+          "../#{@extconf}",
+          "--with-a", # config_options
+          "--with-c", "--with-d", # cross_config_options
+          "--", "--with-b" # extra_options
+        ]
+
+        command.should eq(expected)
+      end
+
+      it "handles spaces in arguments correctly" do
+        @ext.extra_options << "--with-spaces='a b'"
+
+        command = @ext.make_makefile_cmd(@root_path, @tmp_path, @extconf, nil)
+
+        expected_includes = [".", "/injected/include1", "/injected/include2"].join(File::PATH_SEPARATOR)
+        expected = [
+          Gem.ruby,
+          "-I#{expected_includes}",
+          "../#{@extconf}",
+          "--with-a", # config_options
+          "--", "--with-b", "--with-spaces='a b'", # extra_options
+        ]
+
+        command.should eq(expected)
+      end
+
+      it "handles 'nil' in the command array gracefully" do
+        @ext.config_options << nil # imagine this is ENV['NONEXISTENT_VALUE']
+
+        command = @ext.make_makefile_cmd(@root_path, @tmp_path, @extconf, nil)
+
+        expected_includes = [".", "/injected/include1", "/injected/include2"].join(File::PATH_SEPARATOR)
+        expected = [
+          Gem.ruby,
+          "-I#{expected_includes}",
+          "../#{@extconf}",
+          "--with-a", # config_options
+          "--", "--with-b", # extra_options
+        ]
+
+        command.should eq(expected)
+      end
+    end
+
     context '(one extension whose name with directory prefixes)' do
       before :each do
         allow(Rake::FileList).to receive(:[]).and_return(["ext/prefix1/prefix2/extension_one/source.c"], [])
@@ -385,7 +486,7 @@ describe Rake::ExtensionTask do
           receive(:find).with(@ruby_ver, @platform).and_return(nil)
         )
 
-        out, err = capture_output do
+        _, err = capture_output do
           Rake::ExtensionTask.new('extension_one') do |ext|
             ext.cross_compile = true
           end
@@ -507,16 +608,16 @@ describe Rake::ExtensionTask do
           Gem::Requirement.new([">= 1.8", "< 2.12.dev"]),
           Gem::Requirement.new([">= 1.8", "< 2.11.dev"]),
         ]
-        cross_specs.collect(&:required_ruby_version).should == expected_required_ruby_versions
-        cross_specs.collect(&:extensions).should == [[], []]
-        cross_specs.collect(&:platform).collect(&:to_s).should == platforms
-        cross_specs.collect{|s| s.metadata['allowed_push_host']}.should == ['http://test', 'http://test']
+        cross_specs.collect(&:required_ruby_version).should eq expected_required_ruby_versions
+        cross_specs.collect(&:extensions).should eq [[], []]
+        cross_specs.collect(&:platform).collect(&:to_s).should eq platforms
+        cross_specs.collect{|s| s.metadata['allowed_push_host']}.should eq ['http://test', 'http://test']
 
         # original gemspec should keep unchanged
-        spec.required_ruby_version.should == Gem::Requirement.new([">= 0"])
-        spec.platform.should == Gem::Platform::RUBY
-        spec.extensions.should == ['ext/somegem/extconf.rb']
-        spec.metadata['allowed_push_host'].should == 'http://test'
+        spec.required_ruby_version.should eq Gem::Requirement.new([">= 0"])
+        spec.platform.should eq Gem::Platform::RUBY
+        spec.extensions.should eq ['ext/somegem/extconf.rb']
+        spec.metadata['allowed_push_host'].should eq 'http://test'
       end
 
       after :each do
@@ -580,7 +681,7 @@ describe Rake::ExtensionTask do
 
         context 'compile:universal-unknown' do
           it "should be defined" do
-            Rake::Task.task_defined?('compile:universal-unknown').should == true
+            Rake::Task.task_defined?('compile:universal-unknown').should be true
           end
 
           it "should depend on 'compile:extension_one:universal-unknown'" do
@@ -590,7 +691,7 @@ describe Rake::ExtensionTask do
 
         context 'native:universal-unknown' do
           it "should be defined" do
-            Rake::Task.task_defined?('native:universal-unknown').should == true
+            Rake::Task.task_defined?('native:universal-unknown').should be true
           end
 
           it "should depend on 'native:my_gem:universal-unknown'" do
